package net.sourceforge.btrummer.nougatbytes.controller;

import net.sourceforge.btrummer.nougatbytes.model.Color;
import net.sourceforge.btrummer.nougatbytes.state.GameState;
import net.sourceforge.btrummer.nougatbytes.model.Team;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import static net.sourceforge.btrummer.nougatbytes.model.TeamBuilder.team;
import static org.springframework.web.bind.annotation.RequestMethod.GET;
import static org.springframework.web.bind.annotation.RequestMethod.POST;

@Controller
public class GameController {

    private static final Logger LOG = LoggerFactory.getLogger(GameController.class);

    @Autowired
    private GameState state;

    @RequestMapping(path = "/game/start", method = POST)
    public String start(@RequestParam("team0Name") String team0Name,
                        @RequestParam("team0Color") Color team0Color,
                        @RequestParam("team1Name") String team1Name,
                        @RequestParam("team1Color") Color team1Color,
                        @ModelAttribute("model") ModelMap model) {
        LOG.info("Starting new game (team 1: {}, team 2: {})", team0Name, team1Name);

        state.init(
                team(0).withName(team0Name).withColor(team0Color).build(),
                team(1).withName(team1Name).withColor(team1Color).build()
        );

        return "redirect:/game/picture";
    }

    @RequestMapping(path = "/game/score", method = POST)
    public String score(@RequestParam("teamId") int teamId,
                        @RequestParam("points") int points,
                        @ModelAttribute("model") ModelMap model) {
        Team team = state.getTeam(teamId);
        LOG.info("Team {} scores {} points", team.getName(), points);

        team.addScore(points);

        return nextPicture();
    }

    @RequestMapping(path = "/game/skip", method = POST)
    public String skip(@ModelAttribute("model") ModelMap model) {
        LOG.info("Skipping current picture");

        return nextPicture();
    }

    private String nextPicture() {
        return "redirect:/game/" + (state.nextPicture() ? "picture" : "result");
    }

    @RequestMapping(path = "/game/picture", method = GET)
    public String picture(@ModelAttribute("model") ModelMap model) {
        LOG.info("Showing picture {}", state.getCurrentPictureId());

        model.put("picture", state.getCurrentPicture());
        model.put("numberOfPictures", state.getNumberOfPictures());
        model.put("teams", state.getTeams());
        model.put("startingTeam", state.getStartingTeam());

        return "picture";
    }

    @RequestMapping(path = "/game/result", method = GET)
    public String result(@ModelAttribute("model") ModelMap model) {
        LOG.info("Showing results");

        model.put("teams", state.getTeams());
        model.put("winnerTeamId", state.getWinnerTeam() != null ? state.getWinnerTeam().getId() : -1);

        return "result";
    }
}
