package net.sourceforge.btrummer.nougatbytes.state;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import net.sourceforge.btrummer.nougatbytes.model.Picture;
import net.sourceforge.btrummer.nougatbytes.model.Section;
import net.sourceforge.btrummer.nougatbytes.model.Team;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;
import org.springframework.util.StreamUtils;

import java.io.IOException;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import static java.util.Arrays.asList;

@Scope(ConfigurableBeanFactory.SCOPE_SINGLETON)
@Component
public class GameState {

    private final List<Team> teams = new ArrayList<>();

    private final List<Picture> pictures = new ArrayList<>();
    private int currentPictureId;

    public List<Team> getTeams() {
        return teams;
    }

    public Team getTeam(int id) {
        return teams.get(id);
    }

    public Team getStartingTeam() {
        return getTeam(currentPictureId % teams.size());
    }

    public Team getWinnerTeam() {
        int maxScore = teams.stream()
                .map(Team::getScore)
                .max(Comparator.comparingInt(s -> s))
                .get();

        List<Team> winners = teams.stream()
                .filter(t -> t.getScore() == maxScore)
                .collect(Collectors.toList());

        return winners.size() == 1 ? winners.get(0) : null;
    }

    public void init(Team... teams) {
        this.teams.clear();
        this.teams.addAll(asList(teams));

        try {
            ObjectMapper objectMapper = new ObjectMapper();

            List<Section> sections = new ArrayList<>();
            sections.addAll(objectMapper.readValue(
                    StreamUtils.copyToString(
                            getClass().getResourceAsStream("/index.json"),
                            Charset.forName("UTF-8")),
                    new TypeReference<List<Section>>() {
                    })
            );

            pictures.clear();
            int pictureId = 0;
            for (Section section : sections) {
                for (Picture picture : section.getPictures()) {
                    picture.setId(pictureId++);
                    picture.setSection(section);
                    pictures.add(picture);
                }
            }

            currentPictureId = 0;
        } catch (IOException e) {
            throw new RuntimeException("Error reading picture index", e);
        }
    }

    public int getCurrentPictureId() {
        return currentPictureId;
    }

    public Picture getCurrentPicture() {
        return pictures.get(currentPictureId);
    }

    public int getNumberOfPictures() {
        return pictures.size();
    }

    public boolean nextPicture() {
        return ++currentPictureId < pictures.size();
    }
}
